package com.akamai.osmf.samples
{
    import com.akamai.osmf.samples.presenter.BaseControlBarPresenter;
    import com.akamai.osmf.samples.view.BaseVideoView;
    
    import flash.display.Sprite;
    import flash.display.StageAlign;
    import flash.display.StageScaleMode;
    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.events.SecurityErrorEvent;
    import flash.net.URLLoader;
    import flash.net.URLRequest;
    import flash.utils.getDefinitionByName;
    
    import org.osmf.events.MediaFactoryEvent;
    import org.osmf.media.MediaResourceBase;
    import org.osmf.media.PluginInfoResource;
    import org.osmf.media.URLResource;
    import org.osmf.utils.OSMFSettings;
    
    public class BaseSample extends Sprite
    {
		// Variables with getters/setters
        private var _view:BaseVideoView;
		private var _pluginURL:String;
		// Variables with no getters/setters
        private var controlBarPresenter:BaseControlBarPresenter;

		private static const CONFIG_FILE:String = "../../../aasp-sample-config.xml";
        
        public function BaseSample()
        {
            super();
            
            addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
        }
		        
        public function get view():BaseVideoView
        {
            return _view;
        }
        
        protected function onAddedToStage(event:Event):void
        {
            removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			addView();
            setStageProperties();
			loadConfig();
        }
		
		/**
		 * Override in samples to load the plugin from a 
		 * different URL, such as a static load.
		 **/
		protected function get pluginURL():String
		{
			return _pluginURL;
		}
		
		protected function loadConfig():void
		{
			var urlRequest:URLRequest = new URLRequest(CONFIG_FILE);
			var urlLoader:URLLoader = new URLLoader();
			
			setupListeners();
			urlLoader.load(urlRequest);
						
			function setupListeners(add:Boolean=true):void
			{
				if (add)
				{
					urlLoader.addEventListener(Event.COMPLETE, onComplete);
					urlLoader.addEventListener(IOErrorEvent.IO_ERROR, onError);
					urlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, onError);
				}
				else
				{
					urlLoader.removeEventListener(Event.COMPLETE, onComplete);
					urlLoader.removeEventListener(IOErrorEvent.IO_ERROR, onError);
					urlLoader.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, onError);					
				}
			}
			
			function onComplete(event:Event):void
			{
				setupListeners(false);
				
				var xml:XML = new XML(event.target.data);

				var plugins:XMLList = xml.plugins.children();
				for (var i:int = 0; i < plugins.length(); i++)
				{
					if (plugins[i].@id == "AASP")
					{
						_pluginURL = plugins[i].url;
						break;
					}
				}
				
				loadPlugin(pluginURL);
			}
			
			function onError(event:Event):void
			{
				setupListeners(false);

				debug("Unable to load config file: "+event.toString());
			}
		}
		
        protected function setStageProperties():void
        {
            stage.scaleMode = StageScaleMode.NO_SCALE;
            stage.align = StageAlign.TOP_LEFT;
         	
			// remove the player background if the system supports stage video
			if (OSMFSettings.supportsStageVideo)
			{
				_view.background.visible = false;
			}
        }
		
		protected function addView():void
        {
            _view = createView();
            addChild(_view);
        }
        
        /**
         * Override to provide your custom view.
         **/
        protected function createView():BaseVideoView
        {
            return new BaseVideoView();
        }
        
        /**
         * Override to provide your control bar presenter.
         **/
        protected function createControlBarPresenter():BaseControlBarPresenter
        {
            return null;
        }
        
        private function loadPlugin(source:String):void
        {
            var pluginResource:MediaResourceBase;
            
            if (source.search(/^http|^file/i) == -1)
            {
                // This is a class
                var pluginInfoRef:Class = flash.utils.getDefinitionByName(source) as Class;
                pluginResource = new PluginInfoResource(new pluginInfoRef);
            }
            else
            {
                // This is a URL
                pluginResource = new URLResource(source);
            }
            
            setupListeners();
            view.mediaPlayerSprite.mediaFactory.loadPlugin(pluginResource);
            
            function setupListeners(add:Boolean = true):void
            {
                if (add)
                {
                    view.mediaPlayerSprite.mediaFactory.addEventListener(MediaFactoryEvent.PLUGIN_LOAD, onPluginLoad);
                    view.mediaPlayerSprite.mediaFactory.addEventListener(MediaFactoryEvent.PLUGIN_LOAD_ERROR, onPluginLoadError);
                }
                else
                {
                    view.mediaPlayerSprite.mediaFactory.removeEventListener(MediaFactoryEvent.PLUGIN_LOAD, onPluginLoad);
                    view.mediaPlayerSprite.mediaFactory.removeEventListener(MediaFactoryEvent.PLUGIN_LOAD_ERROR, onPluginLoadError);
                }
            }
            
            function onPluginLoad(event:MediaFactoryEvent):void
            {
                debug("plugin loaded successfully: " + pluginResource);
                setupListeners(false);
                
                controlBarPresenter = createControlBarPresenter();
            }
            
            function onPluginLoadError(event:MediaFactoryEvent):void
            {
                debug("plugin failed to load: " + pluginURL);
                setupListeners(false);
            }
        }
    
		private function debug(msg:String):void
		{
			trace(msg);
		}
    }
}
